'
'
' ####################
' #####  PROLOG  #####
' ####################
'
' Demo of creating a path, adding elements, drawing path, flattening path,
' and redrawing the flattened path. Example by Jos Roca.
'
PROGRAM	"flattenpath"
VERSION	"0.0001"
CONSOLE
'
	IMPORT	"xst"   		' standard library	: required by most programs
	IMPORT  "xsx"				' extended std library
'	IMPORT	"xio"				' console io library
	IMPORT	"gdi32"     ' gdi32.dll
	IMPORT  "user32"    ' user32.dll
	IMPORT  "kernel32"  ' kernel32.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library
	IMPORT  "gdiplus"

'TYPE POINTF
'	SINGLE .x
'	SINLGE .y
'END TYPE
'
DECLARE FUNCTION Entry            ()
DECLARE FUNCTION WndProc          (hwnd, msg, wParam, lParam)
DECLARE FUNCTION InitGui          ()
DECLARE FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$)
DECLARE FUNCTION CreateWindows    ()
DECLARE FUNCTION NewWindow        (className$, titleBar$, style, x, y, w, h, exStyle)
DECLARE FUNCTION MessageLoop      ()
DECLARE FUNCTION CleanUp          ()
DECLARE FUNCTION OnPaint          (hdc)
DECLARE FUNCTION ARGB             (UBYTE alpha, UBYTE r, UBYTE g, UBYTE b)
'
'
'
' ######################
' #####  Entry ()  #####
' ######################
'
FUNCTION Entry ()

	STATIC	entry
'
	IF entry THEN RETURN							' enter once
	entry =  $$TRUE										' enter occured

'	XioCreateConsole (title$, 500)		' create console, if console is not wanted, comment out this line
	InitGui ()												' initialize program and libraries
	IF CreateWindows () THEN QUIT (0)	' create windows and other child controls
	MessageLoop ()										' the main message loop
	CleanUp ()												' unregister all window classes
'	XioFreeConsole ()									' free console

END FUNCTION
'
'
' ########################
' #####  WndProc ()  #####
' ########################
'
FUNCTION WndProc (hWnd, msg, wParam, lParam)

	PAINTSTRUCT ps

	SELECT CASE msg

		CASE $$WM_DESTROY :
			PostQuitMessage(0)

		CASE $$WM_PAINT :
			hdc = BeginPaint (hWnd, &ps)
			OnPaint (hdc)
			EndPaint (hWnd, &ps)

		CASE ELSE :
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)

	END SELECT

END FUNCTION
'
'
' ########################
' #####  InitGui ()  #####
' ########################
'
FUNCTION InitGui ()

	SHARED hInst
	SHARED gdiPlusToken
	GdiplusStartupInput gsi

	hInst = GetModuleHandleA (0) 	' get current instance handle
	IFZ hInst THEN QUIT (0)
	InitCommonControls ()					' initialize comctl32.dll library

	' initialize gdiplus
	gsi.GdiplusVersion = 1
	err = GdiplusStartup (&gdiPlusToken, &gsi, NULL)
	IF err THEN
		err$ = "GdiplusStartup error: " + STRING(err)
		MessageBoxA (0, &err$, &"Gdiplus error", $$MB_OK | $$MB_ICONWARNING)
		QUIT (0)
	END IF

END FUNCTION
'
'
' #################################
' #####  RegisterWinClass ()  #####
' #################################
'
FUNCTION RegisterWinClass (className$, addrWndProc, icon$, menu$)

	SHARED hInst
	WNDCLASS wc

	wc.style           = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc     = addrWndProc
	wc.cbClsExtra      = 0
	wc.cbWndExtra      = 0
	wc.hInstance       = hInst
	wc.hIcon           = LoadIconA (hInst, &icon$)
	wc.hCursor         = LoadCursorA (0, $$IDC_ARROW)
	wc.hbrBackground   = GetStockObject ($$WHITE_BRUSH) ' $$COLOR_BTNFACE + 1
	wc.lpszMenuName    = &menu$
	wc.lpszClassName   = &className$

	IFZ RegisterClassA (&wc) THEN RETURN ($$TRUE)

END FUNCTION
'
'
' ##############################
' #####  CreateWindows ()  #####
' ##############################
'
FUNCTION CreateWindows ()

' register window class
	className$  = "GdiPlusFlattenPath"
	addrWndProc = &WndProc()
	icon$ 			= "scrabble"
	menu$ 			= ""
	IF RegisterWinClass (@className$, addrWndProc, @icon$, @menu$) THEN RETURN ($$TRUE)

' create main window
	titleBar$ = "Gdiplus Flatten Path Demo"
	style 		= $$WS_OVERLAPPEDWINDOW
	w 				= 300
	h 				= 300
	exStyle		= 0
	#winMain = NewWindow (@className$, @titleBar$, style, x, y, w, h, exStyle)
	IFZ #winMain THEN RETURN ($$TRUE)

	XstCenterWindow (#winMain)							' center window position
	ShowWindow (#winMain, $$SW_SHOWNORMAL)	' show window

END FUNCTION
'
'
' ##########################
' #####  NewWindow ()  #####
' ##########################
'
FUNCTION NewWindow (className$, titleBar$, style, x, y, w, h, exStyle)

	SHARED hInst

	RETURN CreateWindowExA (exStyle, &className$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)

END FUNCTION

'
' ############################
' #####  MessageLoop ()  #####
' ############################
'
FUNCTION MessageLoop ()

	MSG msg

' main message loop

	IF LIBRARY(0) THEN RETURN								' main program executes message loop

	DO																			' the message loop
		ret = GetMessageA (&msg, NULL, 0, 0)	' retrieve next message from queue

		SELECT CASE ret
			CASE  0 : RETURN msg.wParam					' WM_QUIT message
			CASE -1 : RETURN $$TRUE							' error
			CASE ELSE:
				hwnd = GetActiveWindow ()
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &msg)) THEN		' send only non-dialog messages
  				TranslateMessage (&msg)					' translate virtual-key messages into character messages
  				DispatchMessageA (&msg)					' send message to window callback function WndProc()
				END IF
		END SELECT
	LOOP

END FUNCTION
'
'
' ########################
' #####  CleanUp ()  #####
' ########################
'
FUNCTION CleanUp ()

	SHARED hInst, className$
	SHARED gdiPlusToken

	UnregisterClassA (&className$, hInst)
	GdiplusShutdown (gdiplusToken)

END FUNCTION
'
' #####################
' #####  OnPaint  #####
' #####################
'
' Paint window
'
FUNCTION OnPaint (hdc)

	PathData pData
	POINTF pts[], DataPoints[]
	UBYTE DataTypes[]
	DIM pts[3]

	hStatus = GdipCreateFromHDC (hdc, &pGraphics)

	hStatus = GdipCreatePath ($$FillModeAlternate, &pPath)
	pts[0].x = 20 : pts[0].y = 50
	pts[1].x = 40 : pts[1].y = 70
	pts[2].x = 60 : pts[2].y = 10
	pts[3].x = 80 : pts[3].y = 50
	hStatus = GdipAddPathCurve (pPath, &pts[], 4)
	hStatus = GdipAddPathEllipse (pPath, 20, 100, 150, 80)
	hStatus = GdipAddPathBezier (pPath, 20, 200, 20, 250, 50, 210, 100, 260)

	' Draw the path before flattening.
	hStatus = GdipCreatePen1 (ARGB(255, 255, 0, 0), 1, $$UnitWorld, &pRedPen)
	hStatus = GdipDrawPath (pGraphics, pRedPen, pPath)

	' Flatten the path.
	hStatus = GdipFlattenPath (pPath, NULL, 8)

	' Draw the flattened path.
	hStatus = GdipCreatePen1 (ARGB(255, 0, 0, 255), 1, $$UnitWorld, &pBluePen)
	hStatus = GdipDrawPath (pGraphics, pBluePen, pPath)

	' Create a solid brush.
	hStatus = GdipCreateSolidFill (ARGB(255, 255, 0, 0), &pBrush)

	' Get the path data from the flattened path.
	' We have to allocate the needed arrays.
	' GdipGetPathData fills them with the data.
	hStatus = GdipGetPointCount (pPath, &count)

	IF count THEN
		DIM DataPoints[count - 1]
		DIM DataTypes[count - 1]
		pData.Count = count
		pData.pPoint = &DataPoints[]
		pData.Types = &DataTypes[]
		hStatus = GdipGetPathData (pPath, pData)
	END IF

	' Draw the data points of the flattened path.
	FOR i = 0 TO pData.Count - 1
	hStatus = GdipFillEllipse (pGraphics, pBrush, DataPoints[i].x - 3, DataPoints[i].y - 3, 6, 6)
	NEXT

	' Cleanup
	IF pBrush THEN GdipDeleteBrush (pBrush)
	IF pRedPen THEN GdipDeletePen (pRedPen)
	IF pBluePen THEN GdipDeletePen (pBluePen)
	IF pPath THEN GdipDeletePath (pPath)
	IF pGraphics THEN GdipDeleteGraphics (pGraphics)

END FUNCTION
'
' ##################
' #####  ARGB  #####
' ##################
'
' Return 32-bit color value form argb values
'
FUNCTION ARGB (UBYTE alpha, UBYTE r, UBYTE g, UBYTE b)

	RETURN alpha << 24 + r << 16 + g << 8 + b

END FUNCTION
END PROGRAM
